@@ -6,16 +6,18 @@ from hy import models
 
 from .builtins import parse_builtin
 from .core import (parse_attribute, parse_call, parse_contract,
-                   parse_declaration, parse_declarations, parse_fn, parse_tuple)
+                   parse_declarations, parse_do_body, parse_fn, parse_struct, parse_subscript, parse_tuple)
 from .ops import (BIN_FUNCS, BOOL_OPS, COMP_FUNCS, UNARY_OPS, parse_binop,
                   parse_boolop, parse_comparison, parse_unary)
 from .stmt import parse_assignment, parse_if, parse_return
-from .utils import next_nodeid
+from .utils import next_node_id_maker, next_nodeid
 
 BUILTIN_FUNCS = BIN_FUNCS + COMP_FUNCS + UNARY_OPS + BOOL_OPS
 
 NAME_CONSTS = ["True", "False"]
 
+MACROS = []
+
 def parse_expr(expr):
 
     match expr:
@@ -24,6 +26,8 @@ def parse_expr(expr):
                 value_node = parse_node(models.Symbol("String"))
             elif str(name) == "bytes":
                 value_node = parse_node(models.Symbol("Bytes"))
+            else:
+                value_node = parse_node(models.Symbol(str(name)))
             annotation = vy_nodes.Subscript(ast_type='Subscript', node_id=next_nodeid(), slice=vy_nodes.Index(ast_type='Index', node_id=next_nodeid(), value=parse_node(length)), value=value_node)
             annotation._children.add(value_node)
             return annotation
@@ -43,12 +47,20 @@ def parse_expr(expr):
     match cmd_str:
         case "defcontract":
             return parse_contract(expr)
+        case "defmacro":
+            hy.eval(expr)
+            MACROS.append(str(expr[1]))
+        case str(cmd) if cmd in MACROS:
+            new_node = hy.macroexpand_1(expr)
+            return parse_node(new_node)
         case 'defn':
             return parse_fn(expr)
         case 'return':
             return parse_return(expr)
         case 'quote':
             return parse_tuple(expr)
+        case 'tuple':
+            return parse_tuple(expr)
         case '.':
             return parse_attribute(expr)
         case 'setv':
@@ -56,10 +68,19 @@ def parse_expr(expr):
         case 'if':
             return parse_if(expr)
         case 'defvar':
-            # return parse_declaration(expr[1], expr[2])
             return parse_declarations(expr)
-        case _:
+        case 'defstruct':
+            return parse_struct(expr)
+        case 'subscript':
+            return parse_subscript(expr)
+        case 'get-in':
+            return parse_subscript(expr)
+        case 'public' | 'immutable' | 'constant':
             return parse_call(expr)
+        case 'do':
+            return parse_do_body(expr)
+        case _:
+            return parse_call(expr, wrap_expr=False)
 
 
 def parse_node(node):
@@ -109,16 +130,26 @@ def parse_src(src: str):
             mod_node = ast
         elif isinstance(ast, vy_nodes.VariableDecl):
             vars.append(ast)
+        elif isinstance(ast, vy_nodes.StructDef):
+            vars.append(ast)
         elif isinstance(ast, vy_nodes.FunctionDef):
             fs.append(ast)
         elif isinstance(ast, list):
             for v in ast:
                 vars.append(v)
+        elif ast is None:
+            # macro declarations return None
+            pass
         else:
-            raise(f"Unrecognized top-level form {element} {ast}")
+            raise Exception(f"Unrecognized top-level form {element} {ast}")
 
     for e in vars + fs:
         mod_node.add_to_body(e)
 
 
     return mod_node
+
+with open("dasy/parser/macros.hy", encoding="utf-8") as f:
+    code = f.read()
+    for expr in hy.read_many(code):
+        parse_node(expr)
